@@ -10,8 +10,8 @@
 
 typedef enum
 {
-	RBITMAP_OUTPUT_ARRAY,			    /* output as int array */
-	RBITMAP_OUTPUT_BYTEA				/* output as bytea */
+    RBITMAP_OUTPUT_ARRAY,                /* output as int array */
+    RBITMAP_OUTPUT_BYTEA                 /* output as bytea */
 }RBITMAPOutputFormat;
 
 static const struct config_enum_entry output_format_options[] =
@@ -21,27 +21,37 @@ static const struct config_enum_entry output_format_options[] =
     {NULL, 0, false}
 };
 
-static int	rbitmap_output_format;		/* output format */
+static int    rbitmap_output_format = RBITMAP_OUTPUT_BYTEA;        /* output format */
 
-void		_PG_init(void);
+static roaring_memory_t rb_memory_hook = {
+    .malloc = palloc,
+    .realloc = pg_realloc,
+    .calloc = pg_calloc,
+    .free = pg_free,
+    .aligned_malloc = pg_aligned_malloc,
+    .aligned_free = pg_aligned_free,
+};
+
+void        _PG_init(void);
 /*
  * Module load callback
  */
 void
 _PG_init(void)
 {
-	/* Define custom GUC variables. */
-	DefineCustomEnumVariable("roaringbitmap.output_format",
-							 "Selects output format of roaringbitmap.",
-							 NULL,
-							 &rbitmap_output_format,
-							 RBITMAP_OUTPUT_BYTEA,
-							 output_format_options,
-							 PGC_USERSET,
-							 0,
-							 NULL,
-							 NULL,
-							 NULL);
+    /* Define custom GUC variables. */
+    DefineCustomEnumVariable("roaringbitmap.output_format",
+                             "Selects output format of roaringbitmap.",
+                             NULL,
+                             &rbitmap_output_format,
+                             RBITMAP_OUTPUT_BYTEA,
+                             output_format_options,
+                             PGC_USERSET,
+                             0,
+                             NULL,
+                             NULL,
+                             NULL);
+    roaring_init_memory_hook(rb_memory_hook);
 }
 
 
@@ -89,15 +99,30 @@ Datum
 rb_from_bytea(PG_FUNCTION_ARGS) {
     bytea *serializedbytes = PG_GETARG_BYTEA_P(0);
     roaring_bitmap_t *r1;
+    size_t expectedsize;
+    bytea *serializedbytes2;
+    const char *reason;
 
-    r1 = roaring_bitmap_portable_deserialize(VARDATA(serializedbytes));
+    r1 = roaring_bitmap_portable_deserialize_safe(VARDATA(serializedbytes), VARSIZE(serializedbytes) - VARHDRSZ);
     if (!r1)
         ereport(ERROR,
                 (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                  errmsg("bitmap format is error")));
 
+    if(!roaring_bitmap_internal_validate(r1, &reason)) {
+        roaring_bitmap_free(r1);
+        ereport(ERROR,
+                (errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+                 errmsg("bitmap format is error: %s", reason)));
+    }
+
+    expectedsize = roaring_bitmap_portable_size_in_bytes(r1);
+    serializedbytes2 = (bytea *) palloc(VARHDRSZ + expectedsize);
+    roaring_bitmap_portable_serialize(r1, VARDATA(serializedbytes2));
     roaring_bitmap_free(r1);
-    PG_RETURN_BYTEA_P(serializedbytes);
+
+    SET_VARSIZE(serializedbytes2, VARHDRSZ + expectedsize);
+    PG_RETURN_BYTEA_P(serializedbytes2);
 }
 
 
@@ -114,24 +139,37 @@ roaringbitmap_in(PG_FUNCTION_ARGS) {
     size_t expectedsize;
     bytea *serializedbytes;
     Datum dd;
+    const char *reason;
 
     if(*ptr == '\\' && *(ptr+1) == 'x') {
        /* bytea input */
         dd = DirectFunctionCall1(byteain, PG_GETARG_DATUM(0));
 
         serializedbytes = DatumGetByteaP(dd);
-        r1 = roaring_bitmap_portable_deserialize(VARDATA(serializedbytes));
+        r1 = roaring_bitmap_portable_deserialize_safe(VARDATA(serializedbytes), VARSIZE(serializedbytes) - VARHDRSZ);
         if (!r1)
             ereport(ERROR,
                     (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                      errmsg("bitmap format is error")));
 
+        if(!roaring_bitmap_internal_validate(r1, &reason)) {
+            roaring_bitmap_free(r1);
+            ereport(ERROR,
+                    (errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
+                     errmsg("bitmap format is error: %s", reason)));
+        }
+
+        expectedsize = roaring_bitmap_portable_size_in_bytes(r1);
+        serializedbytes = (bytea *) palloc(VARHDRSZ + expectedsize);
+        roaring_bitmap_portable_serialize(r1, VARDATA(serializedbytes));
         roaring_bitmap_free(r1);
-        return dd;
+
+        SET_VARSIZE(serializedbytes, VARHDRSZ + expectedsize);
+        PG_RETURN_BYTEA_P(serializedbytes);
     }
     /* else int array input */
 
-	/* Find the head char '{' */
+    /* Find the head char '{' */
     while (*ptr && isspace((unsigned char) *ptr))
             ptr++;
     if (*ptr !='{')
@@ -244,14 +282,14 @@ roaringbitmap_out(PG_FUNCTION_ARGS) {
 
     appendStringInfoChar(&buf, '{');
 
-    roaring_init_iterator(r1, &iterator);
+    roaring_iterator_init(r1, &iterator);
     if(iterator.has_value) {
         appendStringInfo(&buf, "%d", (int)iterator.current_value);
-        roaring_advance_uint32_iterator(&iterator);
+        roaring_uint32_iterator_advance(&iterator);
 
         while(iterator.has_value) {
             appendStringInfo(&buf, ",%d", (int)iterator.current_value);
-            roaring_advance_uint32_iterator(&iterator);
+            roaring_uint32_iterator_advance(&iterator);
         }
     }
 
@@ -1308,23 +1346,23 @@ rb_shiftright(PG_FUNCTION_ARGS) {
                      errmsg("failed to create bitmap")));
         }
 
-        roaring_init_iterator(r1, &iterator);
+        roaring_iterator_init(r1, &iterator);
         if(distance > 0){
             while(iterator.has_value) {
                 value = iterator.current_value + distance;
                 if(value >= MAX_BITMAP_RANGE_END)
                     break;
                 roaring_bitmap_add(r2, (uint32)value);
-                roaring_advance_uint32_iterator(&iterator);
+                roaring_uint32_iterator_advance(&iterator);
             }
         }else{
-            roaring_move_uint32_iterator_equalorlarger(&iterator, -distance);
+            roaring_uint32_iterator_move_equalorlarger(&iterator, -distance);
             while(iterator.has_value) {
                 value = iterator.current_value + distance;
                 if(value >= MAX_BITMAP_RANGE_END)
                     break;
                 roaring_bitmap_add(r2, (uint32)value);
-                roaring_advance_uint32_iterator(&iterator);
+                roaring_uint32_iterator_advance(&iterator);
             }
         }
         roaring_bitmap_free(r1);
@@ -1377,13 +1415,13 @@ rb_range(PG_FUNCTION_ARGS) {
                  errmsg("failed to create bitmap")));
     }
 
-    roaring_init_iterator(r1, &iterator);
-    roaring_move_uint32_iterator_equalorlarger(&iterator, rangestart);
+    roaring_iterator_init(r1, &iterator);
+    roaring_uint32_iterator_move_equalorlarger(&iterator, rangestart);
     while(iterator.has_value) {
         if(iterator.current_value >= rangeend)
             break;
         roaring_bitmap_add(r2, iterator.current_value);
-        roaring_advance_uint32_iterator(&iterator);
+        roaring_uint32_iterator_advance(&iterator);
     }
 
     expectedsize = roaring_bitmap_portable_size_in_bytes(r2);
@@ -1424,13 +1462,13 @@ rb_range_cardinality(PG_FUNCTION_ARGS) {
                  errmsg("bitmap format is error")));
 
     card1 = 0;
-    roaring_init_iterator(r1, &iterator);
-    roaring_move_uint32_iterator_equalorlarger(&iterator, rangestart);
+    roaring_iterator_init(r1, &iterator);
+    roaring_uint32_iterator_move_equalorlarger(&iterator, rangestart);
     while(iterator.has_value) {
         if(iterator.current_value >= rangeend)
             break;
         card1++;
-        roaring_advance_uint32_iterator(&iterator);
+        roaring_uint32_iterator_advance(&iterator);
     }
 
     roaring_bitmap_free(r1);
@@ -1480,8 +1518,8 @@ rb_select(PG_FUNCTION_ARGS) {
     }
 
     if (limit > 0) {
-        roaring_init_iterator(r1, &iterator);
-        roaring_move_uint32_iterator_equalorlarger(&iterator, rangestart);
+        roaring_iterator_init(r1, &iterator);
+        roaring_uint32_iterator_move_equalorlarger(&iterator, rangestart);
         if (!reverse) {
             while (iterator.has_value) {
                 if (iterator.current_value >= rangeend
@@ -1490,14 +1528,14 @@ rb_select(PG_FUNCTION_ARGS) {
                 if (count >= offset) {
                     roaring_bitmap_add(r2, iterator.current_value);
                 }
-                roaring_advance_uint32_iterator(&iterator);
+                roaring_uint32_iterator_advance(&iterator);
                 count++;
             }
         } else {
             while (iterator.has_value) {
                 if (iterator.current_value >= rangeend)
                     break;
-                roaring_advance_uint32_iterator(&iterator);
+                roaring_uint32_iterator_advance(&iterator);
                 total_count++;
             }
 
@@ -1506,8 +1544,8 @@ rb_select(PG_FUNCTION_ARGS) {
                 offset = total_count - offset - limit;
                 if(offset < 0)
                     offset = 0;
-                roaring_init_iterator(r1, &iterator);
-                roaring_move_uint32_iterator_equalorlarger(&iterator,rangestart);
+                roaring_iterator_init(r1, &iterator);
+                roaring_uint32_iterator_move_equalorlarger(&iterator,rangestart);
                 count = 0;
                 while (iterator.has_value) {
                     if (iterator.current_value >= rangeend
@@ -1516,7 +1554,7 @@ rb_select(PG_FUNCTION_ARGS) {
                     if (count >= offset) {
                         roaring_bitmap_add(r2, iterator.current_value);
                     }
-                    roaring_advance_uint32_iterator(&iterator);
+                    roaring_uint32_iterator_advance(&iterator);
                     count++;
                 }
             }
@@ -1598,14 +1636,14 @@ rb_to_array(PG_FUNCTION_ARGS)
     {
         out_datums = (Datum *)palloc(sizeof(Datum) * card1);
 
-        iterator = roaring_create_iterator(r1);
+        iterator = roaring_iterator_create(r1);
         while (iterator->has_value)
         {
             out_datums[counter] = Int32GetDatum(iterator->current_value);
             counter++;
-            roaring_advance_uint32_iterator(iterator);
+            roaring_uint32_iterator_advance(iterator);
         }
-        roaring_free_uint32_iterator(iterator);
+        roaring_uint32_iterator_free(iterator);
 
         result = construct_array(out_datums, card1, INT4OID, sizeof(int32), true, 'i');
     }
@@ -1640,7 +1678,7 @@ rb_iterate(PG_FUNCTION_ARGS) {
                     (errcode(ERRCODE_NULL_VALUE_NOT_ALLOWED),
                      errmsg("bitmap format is error")));
 
-        fctx = roaring_create_iterator(r1);
+        fctx = roaring_iterator_create(r1);
 
         funcctx->user_fctx = fctx;
 
@@ -1654,10 +1692,10 @@ rb_iterate(PG_FUNCTION_ARGS) {
     if (fctx->has_value) {
         Datum result;
         result = fctx->current_value;
-        roaring_advance_uint32_iterator(fctx);
+        roaring_uint32_iterator_advance(fctx);
         SRF_RETURN_NEXT(funcctx, result);
     } else {
-        roaring_free_uint32_iterator(fctx);
+        roaring_uint32_iterator_free(fctx);
         SRF_RETURN_DONE(funcctx);
     }
 }
